/*
  Smartcard.cpp - Smartcard library
  Copyright (c) 2012 Fabio Riscica.  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <Smartcard.h>

Smartcard::Smartcard(int slaveSelectPin)
{
  // initialize SPI:
  SPI.begin(); 
  SPI.setDataMode(SPI_MODE0); 
  SPI.setClockDivider(SPI_CLOCK_DIV128); 
 
  // configure slave select pin
  pinMode(slaveSelectPin,OUTPUT);
  digitalWrite(slaveSelectPin,HIGH);
  _slaveSelectPin = slaveSelectPin;
 }

void Smartcard::PowerOnATRSC(unsigned char *atr)
{
  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  SPI.transfer(0x01);
  delay(50);
  atr[0] = SPI.transfer(0x00);
  delay(10);
  atr[1] = SPI.transfer(0x00);
  delay(10);
  atr[2] = SPI.transfer(0x00);
  delay(10);
  atr[3] = SPI.transfer(0x00);
  delay(10);
  digitalWrite(_slaveSelectPin,HIGH);
 }

void Smartcard::PowerOffSC(void)
{
  digitalWrite(_slaveSelectPin,LOW);

  delay(10);
  
  SPI.transfer(0x00);

  delay(10);

  digitalWrite(_slaveSelectPin,HIGH);
 }

unsigned char Smartcard::ReadByte4442(unsigned char address)
 {
  unsigned char data;

  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  SPI.transfer(0x02);
  delay(10);
  SPI.transfer(address);
  delay(10);
  data = SPI.transfer(0x00);
  delay(10);
  digitalWrite(_slaveSelectPin,HIGH);
  
  return data;
   }
   
unsigned char Smartcard::ReadByte4428(unsigned int address)
 {
  unsigned char data;

  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  SPI.transfer(0x12);
  delay(10);
  SPI.transfer(address);
  delay(10);
  SPI.transfer(address >> 8);
  delay(10);
  data = SPI.transfer(0x00);
  delay(10);
  digitalWrite(_slaveSelectPin,HIGH);
  
  return data;
   }

void Smartcard::WriteByte4442(unsigned char address, unsigned char data)
{
  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  SPI.transfer(0x03);
  delay(10);
  SPI.transfer(address);
  delay(10);
  SPI.transfer(data);
  delay(10);
  digitalWrite(_slaveSelectPin,HIGH);
  }
 
void Smartcard::WriteByte4428(unsigned int address, unsigned char data)
{
  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  SPI.transfer(0x13);
  delay(10);
  SPI.transfer(address);
  delay(10);
  SPI.transfer(address >> 8);
  delay(10);
  SPI.transfer(data);
  delay(10);
  digitalWrite(_slaveSelectPin,HIGH);
  }

unsigned char Smartcard::VerifyPSC4442(unsigned char *psc)
{
  unsigned char counter;
      
  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  SPI.transfer(0x04);
  delay(10);
  SPI.transfer(psc[0]);
  delay(10);
  SPI.transfer(psc[1]);
  delay(10);
  SPI.transfer(psc[2]);
  delay(20);
  counter = SPI.transfer(0x00);
  delay(10);
  digitalWrite(_slaveSelectPin,HIGH);
      
  return counter;
  }

unsigned char Smartcard::VerifyPSC4428(unsigned char *psc)
{
  unsigned char counter;
      
  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  SPI.transfer(0x14);
  delay(10);
  SPI.transfer(psc[0]);
  delay(10);
  SPI.transfer(psc[1]);
  delay(20);
  counter = SPI.transfer(0x00);
  delay(10);
  digitalWrite(_slaveSelectPin,HIGH);
      
  return counter;
  }

void Smartcard::ModifyPSC4442(unsigned char *psc)
{
  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  SPI.transfer(0x05);
  delay(10);
  SPI.transfer(psc[0]);
  delay(10);
  SPI.transfer(psc[1]);
  delay(10);
  SPI.transfer(psc[2]);
  delay(20);
  digitalWrite(_slaveSelectPin,HIGH);
  }

unsigned char Smartcard::IsCardPresent(unsigned char cam)
{
  unsigned char data;
      
  digitalWrite(_slaveSelectPin,LOW);
  delay(10);
  if(cam)
    SPI.transfer(0x06);
  else
    SPI.transfer(0x07);
  delay(10);
  data = SPI.transfer(0x00);
  delay(10);
  digitalWrite(_slaveSelectPin,HIGH);
    
  return data;
  }

unsigned char Smartcard::PowerOnATR(unsigned char cam,unsigned char *atr)
{
  unsigned char i,length;
  
  digitalWrite(_slaveSelectPin,LOW);

  delay(10);
  
  if(cam)
    SPI.transfer(0x08);
  else
    SPI.transfer(0x09);

  delay(1000);
  
  length = SPI.transfer(0x00);
  
  delay(10);

  for(i=0;i<length;i++){
    atr[i] = SPI.transfer(0x00);
    delay(10);
  }

  digitalWrite(_slaveSelectPin,HIGH);

  return length;
 }
 
 void Smartcard::ApduCommand(unsigned char cam, unsigned char T, unsigned char *C,unsigned char *D,unsigned char *SW)
 {
  unsigned char i;
       
  digitalWrite(_slaveSelectPin,LOW);

  delay(10);
  
  if(T)
  {
    if(cam)
      SPI.transfer(0x0D);
        else
      SPI.transfer(0x0E);
  }  
  else
  {
    if(cam)
      SPI.transfer(0x0A);
        else
      SPI.transfer(0x0B);
  }
  SPI.transfer(C[0]);
  delay(10);
  SPI.transfer(C[1]);
  delay(10);
  SPI.transfer(C[2]);
  delay(10);
  SPI.transfer(C[3]);
  delay(10);
  SPI.transfer(C[4]);
  delay(10);
  SPI.transfer(C[5]);
  delay(10);
  
  for(i=0;i<C[4];i++){
    SPI.transfer(D[i]);
    delay(10);
    }
    
  delay(150);
  
  for(i=0;i<C[5];i++){
    D[i] = SPI.transfer(0x00);
    delay(10);
  }
 
  SW[0] = SPI.transfer(0x00);
  delay(10);
  SW[1] = SPI.transfer(0x00);
  delay(10);
  
  digitalWrite(_slaveSelectPin,HIGH);
 }

unsigned char Smartcard::Shutdown(unsigned char cam)
{
  digitalWrite(_slaveSelectPin,LOW);

  delay(10);
  
    if(cam)
      SPI.transfer(0x0C);
        else
      SPI.transfer(0x0F);

  delay(10);

  digitalWrite(_slaveSelectPin,HIGH);
 }
