/*
  Smartcard Library - ApduCommand
 
 Demonstrates the use of a CPU smartcard.  The Smartcard
 library works with the Arduino Smartcard shield and was tested
 with PAYFLEX and ACOS smartcards.
 
 This sketch demonstrates the use of the ApduCommand() function.
 
 The circuit:
 SmartShield (http://www.elettroinnova.it)
 
 Library originally added 12 Apr 2012
 by Fabio Riscica
  
 This example code is in the public domain.

*/

// include the SPI library:
#include <SPI.h>

//include the Smartcard library
#include <Smartcard.h>

// initialize the library with address of slave select pin
Smartcard smartcard(7);

//convert from hexadecimal to ASCII for display the result 
void HexToAsc(char *c){
	if (((*c&0x0F)>=0)&&((*c&0x0F)<=9))
		*(c+1) = (*c&0x0F)+0x30;
	else
		*(c+1) = (*c&0x0F)+0x37;

	if ((((*c&0xF0)>>4)>=0)&&(((*c&0xF0)>>4)<=9))
		*c = ((*c&0xF0)>>4)+0x30;
	else
		*c = ((*c&0xF0)>>4)+0x37;
}

void setup() {
  // start the serial library:
  Serial.begin(9600);
}

void loop() {
      unsigned char data[32];
      char ascii_data[2];
      unsigned char C[6];
      unsigned char D[8];
      unsigned char SW[2];
      unsigned char counter;
      unsigned char i, length;
     
      Serial.print("Wait smartcard insertion... ");
      
      //wait smartcard
      while(!smartcard.IsCardPresent(1)){
        delay(500);  
      }
      Serial.println("done");
 
       //reset smartcard
      counter = 0;
      do
      {
          length = smartcard.PowerOnATR(1,data);
          counter++;
      }
      while((counter < 10)&&(data[0] != 0x3B));
      
      Serial.println();
 
      Serial.println(">> ApduCommand <<");
      Serial.println();
      Serial.println("-> 00 A4 00 00 02 00 3F 00");
      
      //apdu command
      C[0] = 0x00;
      C[1] = 0xA4;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x02;
      C[5] = 0x00;
      
      D[0] = 0x3F;
      D[1] = 0x00;
      
      counter = 0;
      do {
          smartcard.ApduCommand(1,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      
      if(C[5])
        Serial.print("<- ");
      
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      
      if(C[5])
        Serial.println();
  
      Serial.print("<- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();
      Serial.println();

      Serial.println("-> 00 84 00 00 00 08");
       
      //apdu command
      C[0] = 0x00;
      C[1] = 0x84;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x00;
      C[5] = 0x08;
      
      counter = 0;
      do {
          smartcard.ApduCommand(1,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      
      if(C[5])
        Serial.print("<- ");
      
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      
      if(C[5])
        Serial.println();
  
      Serial.print("<- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();
      Serial.println();
      
      Serial.print("Wait smartcard extraction...");
      
      //wait smartcard extraction
      smartcard.Shutdown(1);
      while(smartcard.IsCardPresent(1)){
        delay(500);  
      }
      Serial.println("done");
      Serial.println();

  }

