/*
  Smartcard Library - Authentication
 
 Demonstrates the use of CPU smartcards.  The Smartcard
 library works with the Arduino Smartcard shield and was tested
 with PAYFLEX and ACOS smartcards.
 
 This sketch demonstrates the use of muthual authentication 
 between a Payflex smartcard and a SAM module. 
 
 The circuit:
 SmartShield (http://www.elettroinnova.it)
 
 Library originally added 13 Apr 2012
 by Fabio Riscica
  
 This example code is in the public domain.

*/

// include the SPI library:
#include <SPI.h>

//include the Smartcard library
#include <Smartcard.h>

// initialize the library with address of slave select pin
Smartcard smartcard(7);

//convert from hexadecimal to ASCII for display the result 
void HexToAsc(char *c){
	if (((*c&0x0F)>=0)&&((*c&0x0F)<=9))
		*(c+1) = (*c&0x0F)+0x30;
	else
		*(c+1) = (*c&0x0F)+0x37;

	if ((((*c&0xF0)>>4)>=0)&&(((*c&0xF0)>>4)<=9))
		*c = ((*c&0xF0)>>4)+0x30;
	else
		*c = ((*c&0xF0)>>4)+0x37;
}

void setup() {
  // start the serial library:
  Serial.begin(9600);
}

void loop() {
      unsigned char data[32];
      char ascii_data[2];
      unsigned char C[6];
      unsigned char D[8];
      unsigned char SW[2];
      unsigned char CHALLENGE[8];
      unsigned char CERTIFICATE[6];
      unsigned char counter;
      unsigned char i, length;
     
      Serial.print("Wait smartcard insertion... ");
      
      //wait smartcard
      while(!smartcard.IsCardPresent(1)){
        delay(500);  
      }
      Serial.println("done");
      Serial.println();
 
       //reset smartcard
      do {
          length = smartcard.PowerOnATR(1,data);
         }
      while((data[0]  != 0x3B)&&
            (data[1]  != 0x69)&&
            (data[2]  != 0x00)&&
            (data[3]  != 0x24)&&
            (data[4]  != 0x94)&&
            (data[5]  != 0x01)&&
            (data[6]  != 0x00)&&
            (data[7]  != 0x00)&&
            (data[8]  != 0x00)&&
            (data[9]  != 0x00)&&
            (data[10] != 0xA1)&&
            (data[11] != 0xA9));
      
      Serial.print("[1] -> ");
  
      for(i=0;i<length;i++){
        ascii_data[0] = data[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      Serial.println();

       //reset SAM
      do {
          length = smartcard.PowerOnATR(0,data);
         }
      while((data[0]  != 0x3B)&&
            (data[1]  != 0x69)&&
            (data[2]  != 0x00)&&
            (data[3]  != 0x24)&&
            (data[4]  != 0x94)&&
            (data[5]  != 0x01)&&
            (data[6]  != 0x00)&&
            (data[7]  != 0x00)&&
            (data[8]  != 0x00)&&
            (data[9]  != 0x00)&&
            (data[10] != 0xA1)&&
            (data[11] != 0xA9));
      
      Serial.print("[0] -> ");
  
      for(i=0;i<length;i++){
        ascii_data[0] = data[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      Serial.println();
  
      //SAM SELECT 7F 10
      Serial.println("[0] -> 00 A4 00 00 02 00 3F 00");
      C[0] = 0x00;
      C[1] = 0xA4;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x02;
      C[5] = 0x00;
      D[0] = 0x7F;
      D[1] = 0x10;
     
      counter = 0;
      do {
          smartcard.ApduCommand(0,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      if(C[5])
        Serial.print("    <- ");
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      if(C[5])
        Serial.println();
      Serial.print("    <- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();
      
      //SAM SELECT 00 01
      Serial.println("[0] -> 00 A4 00 00 02 00 00 01");
      C[0] = 0x00;
      C[1] = 0xA4;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x02;
      C[5] = 0x00;
      D[0] = 0x00;
      D[1] = 0x01;
      
      counter = 0;
      do {
          smartcard.ApduCommand(0,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      if(C[5])
        Serial.print("    <- ");
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      if(C[5])
        Serial.println();
      Serial.print("    <- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();
      
      //SAM GET CHALLENGE
      Serial.println("[0] -> 00 84 00 00 00 08");
      C[0] = 0x00;
      C[1] = 0x84;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x00;
      C[5] = 0x08;
      
      counter = 0;
      do {
          smartcard.ApduCommand(0,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      if(C[5])
        Serial.print("    <- ");
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      if(C[5])
        Serial.println();
      Serial.print("    <- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();
      
      //store challenge
      for(i=0;i<C[5];i++)
        CHALLENGE[i] = D[i];
      
      //SMC SELECT 7F 10
      Serial.println("[1] -> 00 A4 00 00 02 00 3F 00");
      C[0] = 0x00;
      C[1] = 0xA4;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x02;
      C[5] = 0x00;
      D[0] = 0x7F;
      D[1] = 0x10;
     
      counter = 0;
      do {
          smartcard.ApduCommand(1,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      if(C[5])
        Serial.print("    <- ");
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      if(C[5])
        Serial.println();
      Serial.print("    <- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();
      
      //SMC SELECT 00 10
      Serial.println("[1] -> 00 A4 00 00 02 00 00 10");
      C[0] = 0x00;
      C[1] = 0xA4;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x02;
      C[5] = 0x00;
      D[0] = 0x00;
      D[1] = 0x10;
      
      counter = 0;
      do {
          smartcard.ApduCommand(1,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      if(C[5])
        Serial.print("    <- ");
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      if(C[5])
        Serial.println();
      Serial.print("    <- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();
      
       //SMC INTERNAL AUTHENTICATION
      Serial.println("[1] -> 00 88 00 02 08 00");
      C[0] = 0x00;
      C[1] = 0x88;
      C[2] = 0x00;
      C[3] = 0x02;
      C[4] = 0x08;
      C[5] = 0x00;
      D[0] = CHALLENGE[0];
      D[1] = CHALLENGE[1];
      D[2] = CHALLENGE[2];
      D[3] = CHALLENGE[3];
      D[4] = CHALLENGE[4];
      D[5] = CHALLENGE[5];
      D[6] = CHALLENGE[6];
      D[7] = CHALLENGE[7];
      
      counter = 0;
      do {
          smartcard.ApduCommand(1,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      if(C[5])
        Serial.print("    <- ");
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      if(C[5])
        Serial.println();
      Serial.print("    <- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();

      //SMC GET RESPONSE
      Serial.println("[1] -> 00 C0 00 00 00 06");
      C[0] = 0x00;
      C[1] = 0xC0;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x00;
      C[5] = 0x06;
      
      counter = 0;
      do {
          smartcard.ApduCommand(1,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      if(C[5])
        Serial.print("    <- ");
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      if(C[5])
        Serial.println();
      Serial.print("    <- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();
      
      //store certificate
      for(i=0;i<C[5];i++)
        CERTIFICATE[i] = D[i];
      
       //SAM EXTERNAL AUTHENTICATION
      Serial.println("[0] -> 00 82 00 04 06 00");
      C[0] = 0x00;
      C[1] = 0x82;
      C[2] = 0x00;
      C[3] = 0x04;
      C[4] = 0x06;
      C[5] = 0x00;
      D[0] = CERTIFICATE[0];
      D[1] = CERTIFICATE[1];
      D[2] = CERTIFICATE[2];
      D[3] = CERTIFICATE[3];
      D[4] = CERTIFICATE[4];
      D[5] = CERTIFICATE[5];
      
      counter = 0;
      do {
          smartcard.ApduCommand(0,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 1)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      if(C[5])
        Serial.print("    <- ");
      for(i=0;i<C[5];i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);      
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
        Serial.print(" ");
      }
      if(C[5])
        Serial.println();
      Serial.print("    <- ");
      ascii_data[0] = SW[0];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.print(" ");
      ascii_data[0] = SW[1];
      HexToAsc(ascii_data);      
      Serial.print(ascii_data[0]);
      Serial.print(ascii_data[1]);
      Serial.println();

      Serial.println();
      Serial.print("Wait smartcard extraction...");
      
      //wait smartcard extraction
      smartcard.Shutdown(1);
      while(smartcard.IsCardPresent(1)){
        delay(500);  
      }
      smartcard.Shutdown(0);
     
      Serial.println("done");
      Serial.println();

  }

