/*
  Smartcard Library - WriteCard
 
 Demonstrates the use of a SLE4428 sync smartcard.  The Smartcard
 library works with the Arduino Smartcard shield.
 
 This sketch demonstrates the use of the Verify4428() and 
 WriteByte4428() functions. Please check the correct PSC code of
 smartcard, otherwise after eight incorrect attempts the card
 will be blocked. 
 
 The circuit:
 SmartShield (http://www.elettroinnova.it)
 
 Library originally added 12 Apr 2012
 by Fabio Riscica
  
 This example code is in the public domain.

*/

// include the SPI library:
#include <SPI.h>

//include the Smartcard library
#include <Smartcard.h>

// initialize the library with address of slave select pin
Smartcard smartcard(7);

//convert from hexadecimal to ASCII for display the result 
void HexToAsc(char *c){
	if (((*c&0x0F)>=0)&&((*c&0x0F)<=9))
		*(c+1) = (*c&0x0F)+0x30;
	else
		*(c+1) = (*c&0x0F)+0x37;

	if ((((*c&0xF0)>>4)>=0)&&(((*c&0xF0)>>4)<=9))
		*c = ((*c&0xF0)>>4)+0x30;
	else
		*c = ((*c&0xF0)>>4)+0x37;
}

void setup() {
  // start the serial library:
  Serial.begin(9600);
}

void loop() {
      unsigned char data[16], newdata[16];
      char ascii_data[2];
      unsigned char psc[2];
      char i;
    
      Serial.print("Wait smartcard insertion... ");
      
      //wait smartcard
      while(!smartcard.IsCardPresent(1)){
        delay(500);  
      }
      Serial.println("done");
      Serial.println();
 
      //reset smartcard
      smartcard.PowerOnATRSC(data);
      
      Serial.println(">> VerifyPSC4428 <<");
      Serial.println();
      
      //check PSC
      psc[0] = 0xFF;
      psc[1] = 0xFF;
      data[0] = smartcard.VerifyPSC4428(psc);
      
      switch(data[0]){
        case 0xFF :  
                     break;
        case 0xFE :
                     Serial.println("failed - seven attempts remaining");
                     Serial.println();
                     break;
        case 0xFC :
                     Serial.println("failed - six attempts remaining");
                     Serial.println();
                     break;
        case 0xF8 :
                     Serial.println("failed - five attempts remaining");
                     Serial.println();
                     break;
        case 0xF0 :
                     Serial.println("failed - four attempts remaining");
                     Serial.println();
                     break;
        case 0xE0 :
                     Serial.println("failed - three attempts remaining");
                     Serial.println();
                     break;
        case 0xC0 :
                     Serial.println("failed - two attempts remaining");
                     Serial.println();
                     break;
        case 0x80 :
                     Serial.println("failed - one attempts remaining");
                     Serial.println();
                     break;
        case 0x00 :
                     Serial.println("locked");
                     Serial.println();
                     break;                     
      }
      
      Serial.println(">> ReadByte4428 <<");
      Serial.println();
      
      //read bytes from 0x20 to 0x2F
      for (i=0;i<16;i++){
          data[i] = smartcard.ReadByte4428(i+0x20);
          ascii_data[0] = data[i];
          HexToAsc(ascii_data);      
          Serial.print(ascii_data[0]);
          Serial.print(ascii_data[1]);
          Serial.print(" ");
        }
      Serial.println();
      Serial.println();

      Serial.println(">> WriteByte4428 (clear) <<");
      Serial.println();
      
      //clear bytes from 0x20 to 0x2F
      for (i=0;i<16;i++)
          smartcard.WriteByte4428(i+0x20,0x00);

      Serial.println(">> ReadByte4428 <<");
      Serial.println();
      
      //read bytes from 0x20 to 0x2F
      for (i=0;i<16;i++){
          newdata[i] = smartcard.ReadByte4428(i+0x20);
          ascii_data[0] = newdata[i];
          HexToAsc(ascii_data);      
          Serial.print(ascii_data[0]);
          Serial.print(ascii_data[1]);
          Serial.print(" ");
        }
      Serial.println();
      Serial.println();

      Serial.println(">> WriteByte4428 (restore) <<");
      Serial.println();
      
     //restore bytes from 0x20 to 0x2F
      for (i=0;i<16;i++)
          smartcard.WriteByte4428(i+0x20,data[i]);

      Serial.println(">> ReadByte4428 <<");
      Serial.println();

     //read bytes from 0x20 to 0x2F
      for (i=0;i<16;i++){
          newdata[i] = smartcard.ReadByte4428(i+0x20);
          ascii_data[0] = newdata[i];
          HexToAsc(ascii_data);      
          Serial.print(ascii_data[0]);
          Serial.print(ascii_data[1]);
          Serial.print(" ");
      }
      Serial.println();
      Serial.println();

      Serial.print("Wait smartcard extraction...");
      
      smartcard.PowerOffSC();

      //wait smartcard extraction
      while(smartcard.IsCardPresent(1)){
        delay(500);  
      }
      Serial.println("done");
      Serial.println();

  }

