/*
  Smartcard Library - WriteCard
 
 Demonstrates the use of a SLE4442 sync smartcard.  The Smartcard
 library works with the Arduino Smartcard shield.
 
 This sketch demonstrates the use of the VerifyPSC4442() and 
 WriteByte4442() functions. Please check the correct PSC code of
 smartcard, otherwise after three incorrect attempts the card
 will be blocked. 
 
 The circuit:
 SmartShield (http://www.elettroinnova.it)
 
 Library originally added 12 Apr 2012
 by Fabio Riscica
  
 This example code is in the public domain.

*/

// include the SPI library:
#include <SPI.h>

//include the Smartcard library
#include <Smartcard.h>

// initialize the library with address of slave select pin
Smartcard smartcard(7);

//convert from hexadecimal to ASCII for display the result 
void HexToAsc(char *c){
	if (((*c&0x0F)>=0)&&((*c&0x0F)<=9))
		*(c+1) = (*c&0x0F)+0x30;
	else
		*(c+1) = (*c&0x0F)+0x37;

	if ((((*c&0xF0)>>4)>=0)&&(((*c&0xF0)>>4)<=9))
		*c = ((*c&0xF0)>>4)+0x30;
	else
		*c = ((*c&0xF0)>>4)+0x37;
}

void setup() {
  // start the serial library:
  Serial.begin(9600);
}

void loop() {
      unsigned char data[16], newdata[16];
      char ascii_data[2];
      unsigned char psc[3];
      char i;
    
      Serial.print("Wait smartcard insertion... ");
      
      //wait smartcard
      while(!smartcard.IsCardPresent(1)){
        delay(500);  
      }
      Serial.println("done");
      Serial.println();
 
      //reset smartcard
      smartcard.PowerOnATRSC(data);
      
      Serial.println(">> VerifyPSC4442 <<");
      Serial.println();
      
      //check PSC
      psc[0] = 0x17;
      psc[1] = 0x05;
      psc[2] = 0x65;
      data[0] = smartcard.VerifyPSC4442(psc);
      
      switch(data[0]){
        case 0x07 :  
                     break;
        case 0x06 :
                     Serial.println("failed - two attempts remaining");
                     Serial.println();
                     break;
        case 0x04 :
                     Serial.println("failed - one attempt remaining");
                     Serial.println();
                     break;
        case 0x00 :
                     Serial.println("locked");
                     Serial.println();
                     break;                     
      }
      
      Serial.println(">> ReadByte4442 <<");
      Serial.println();
      
      //read bytes from 0x20 to 0x2F
      for (i=0;i<16;i++){
          data[i] = smartcard.ReadByte4442(i+0x20);
          ascii_data[0] = data[i];
          HexToAsc(ascii_data);      
          Serial.print(ascii_data[0]);
          Serial.print(ascii_data[1]);
          Serial.print(" ");
        }
      Serial.println();
      Serial.println();

      Serial.println(">> WriteByte4442 (clear) <<");
      Serial.println();
      
      //clear bytes from 0x20 to 0x2F
      for (i=0;i<16;i++)
          smartcard.WriteByte4442(i+0x20,0x00);

      Serial.println(">> ReadByte4442 <<");
      Serial.println();
      
      //read bytes from 0x20 to 0x2F
      for (i=0;i<16;i++){
          newdata[i] = smartcard.ReadByte4442(i+0x20);
          ascii_data[0] = newdata[i];
          HexToAsc(ascii_data);      
          Serial.print(ascii_data[0]);
          Serial.print(ascii_data[1]);
          Serial.print(" ");
        }
      Serial.println();
      Serial.println();

      Serial.println(">> WriteByte4442 (restore) <<");
      Serial.println();
      
     //restore bytes from 0x20 to 0x2F
      for (i=0;i<16;i++)
          smartcard.WriteByte4442(i+0x20,data[i]);

      Serial.println(">> ReadByte4442 <<");
      Serial.println();

     //read bytes from 0x20 to 0x2F
      for (i=0;i<16;i++){
          newdata[i] = smartcard.ReadByte4442(i+0x20);
          ascii_data[0] = newdata[i];
          HexToAsc(ascii_data);      
          Serial.print(ascii_data[0]);
          Serial.print(ascii_data[1]);
          Serial.print(" ");
      }
      Serial.println();
      Serial.println();

      Serial.print("Wait smartcard extraction...");
      
      smartcard.PowerOffSC();

      //wait smartcard extraction
      while(smartcard.IsCardPresent(1)){
        delay(500);  
      }
      Serial.println("done");
      Serial.println();

  }

